@@ -9,7 +9,7 @@ from pca.pca import (
 
 __author__ = 'Erdogan Tasksen'
 __email__ = 'erdogant@gmail.com'
-__version__ = '1.8.4'
+__version__ = '1.8.6'
 
 # module level doc-string
 __doc__ = """
@@ -18,7 +18,7 @@ pca
 
 Description
 -----------
-pca is a python package to perform Principal Component Analysis and to make insightful plots.
+pca: A Python Package for Principal Component Analysis.
 
 Examples
 --------
@@ -38,7 +38,7 @@ Examples
 >>> # Scree plot together with explained variance.
 >>> fig, ax = model.plot()
 >>>
->>> # Plot loadings.
+>>> # Plot loadings
 >>> fig, ax = model.biplot()
 >>>
 >>> # Plot loadings with outliers
