@@ -2,6 +2,97 @@ from pca import pca
 import pandas as pd
 import numpy as np
 
+
+# %%
+from sklearn.datasets import make_friedman1
+X, _ = make_friedman1(n_samples=200, n_features=30, random_state=0)
+
+# Init
+model = pca()
+# Fit
+model.fit_transform(X)
+
+# Make plot with blue arrows and text
+fig, ax = model.biplot(c=[0,0,0], s=25, fontdict={'size':10, 'weight':'normal'}, color_arrow='blue', title=None, hotellingt2=True, n_feat=10, visible=True)
+
+# Use the existing fig and create new edits such red arrows for the first three loadings. Also change the font sizes.
+fig, ax = model.biplot(c=[0,0,0], s=25, fontdict={'size':16, 'weight':'bold'}, color_arrow='red', n_feat=3, title='updated fig.', visible=True, fig=fig)
+
+
+# %%
+# https://github.com/erdogant/pca/issues/40
+
+# Import iris dataset and other required libraries
+from sklearn.datasets import load_iris
+import pandas as pd
+import matplotlib as mpl
+import colourmap
+from pca import pca
+
+# Class labels
+y = load_iris().target
+
+# Initialize pca
+model = pca(n_components=3, normalize=True)
+# Dataset
+X = pd.DataFrame(index=y, data=load_iris().data, columns=load_iris().feature_names)
+# Fit transform
+out = model.fit_transform(X)
+
+# The default setting is to color on classlabels (y). These are provided as the index in the dataframe.
+model.biplot()
+
+# Use custom cmap for classlabels (as an example I explicitely provide three colors).
+model.biplot(cmap=mpl.colors.ListedColormap(['green', 'red', 'blue']))
+
+# Set custom classlabels. Coloring is based on the input colormap (cmap).
+y[10:15]=4
+model.biplot(y=y, cmap='Set2')
+
+# Set custom classlabels and also use custom colors.
+c = colourmap.fromlist(y, cmap='Set2')[0]
+c[10:15] = [0,0,0]
+model.biplot(y=y, c=c)
+
+
+# Remove scatterpoints by setting cmap=None
+model.biplot(cmap=None)
+
+
+# Color on classlabel (Unchanged)
+model.biplot()
+# Use cmap colors for classlabels (unchanged)
+model.biplot(y=y, cmap=mpl.colors.ListedColormap(['green', 'red', 'blue']))
+# Do not show points when cmap=None (unchanged)
+model.biplot(y=load_iris().target, cmap=None)
+# Plot all points as unique entity (unchanged)
+model.biplot(y=y, gradient='#ffffff', cmap=mpl.colors.ListedColormap(['green', 'red', 'blue']))
+
+
+# %%
+import numpy as np
+from sklearn.datasets import load_iris
+
+# Load dataset
+X = pd.DataFrame(data=load_iris().data, columns=load_iris().feature_names, index=load_iris().target)
+
+# Load pca
+from pca import pca
+model = pca(n_components=0.95)
+model = pca(n_components=3)
+results = model.fit_transform(X, verbose=2)
+
+
+model.scatter()
+model.scatter(gradient='#ffffff')
+model.scatter(gradient='#ffffff', cmap='tab20')
+model.scatter(gradient='#ffffff', cmap='tab20', y=np.ones_like(model.results['PC'].index.values))
+
+model.scatter(gradient='#5dfa02')
+model.scatter(gradient='#5dfa02', cmap='tab20')
+model.scatter(gradient='#5dfa02', cmap='tab20', y=np.ones_like(model.results['PC'].index.values))
+
+
 # %%
 # import pca
 # print(pca.__version__)
@@ -20,8 +111,8 @@ X, _ = make_friedman1(n_samples=200, n_features=30, random_state=0)
 # model = pca(method='sparse_pca')
 model = pca(method='trunc_svd')
 model.fit_transform(X)
-model.plot()
-model.biplot(fontdict={'size':16, 'weight':'bold'})
+model.plot(title=None)
+model.biplot(fontdict={'size':16, 'weight':'bold'}, title=None)
 model.biplot3d()
 model.scatter()
 
