@@ -1,4 +1,4 @@
-"""pca is a python package to perform Principal Component Analysis and to make insightful plots."""
+"""pca: A Python Package for Principal Component Analysis."""
 
 # %% Libraries
 import scatterd as scatterd
@@ -322,7 +322,7 @@ class pca():
         PCnames = list(map(lambda x: ''.join(x), PCzip))
         loadings = pd.DataFrame(loadings, columns=col_labels, index=PCnames)
         # Return
-        return(loadings)
+        return loadings
 
     # Top scoring components
     def compute_topfeat(self, loadings=None, verbose=3):
@@ -453,7 +453,7 @@ class pca():
             # ax2.set_title('Zero-mean with unit variance normalized')
             # ax2.grid(True)
 
-        return(X, row_labels, col_labels, scaler)
+        return (X, row_labels, col_labels, scaler)
 
     # Figure pre processing
     def _fig_preprocessing(self, y, n_feat, d3):
@@ -492,18 +492,21 @@ class pca():
     # Scatter plot
     def scatter3d(self,
                   y=None,
+                  c=None,
+                  s=50,
                   label=True,
                   PC=[0, 1, 2],
                   SPE=False,
                   hotellingt2=False,
                   alpha_transparency=None,
                   gradient=None,
-                  fontdict={'weight': 'normal', 'size': 12, 'ha': 'center', 'va': 'center'},
+                  fontdict={'weight': 'normal', 'size': 12, 'ha': 'center', 'va': 'center', 'c': 'black'},
                   cmap='Set1',
                   title=None,
                   legend=True,
                   figsize=(15, 10),
                   visible=True,
+                  fig=None,
                   verbose=3):
         """Scatter 3d plot.
 
@@ -511,6 +514,13 @@ class pca():
         ----------
         y : array-like, default: None
             Label for each sample. The labeling is used for coloring the samples.
+        c: list/array of RGB colors for each sample.
+            Color of samples in RGB colors.
+            [0,0,0]: If a single color is given, all samples get that color.
+        s: Int or list/array (default: 50)
+            Size(s) of the scatter-points.
+            [20, 10, 50, ...]: In case of list: should be same size as the number of PCs -> .results['PC']
+            50: all points get this size.
         label : Bool, default: True
             True Show the labels.
             False: Do not show the labels
@@ -528,11 +538,14 @@ class pca():
             '#FFFFFF'
         fontdict : dict.
             dictionary containing properties for the arrow font-text
-            {'weight': 'normal', 'size': 10, 'ha': 'center', 'va': 'center'}
+            {'weight': 'normal', 'size': 10, 'ha': 'center', 'va': 'center', 'c': 'black'}
         cmap : String, optional, default: 'Set1'
             Colormap. If set to None, no points are shown.
         title : str, default: None
             Title of the figure.
+            None: Automatically create title text based on results.
+            '' : Remove all title text.
+            'title text' : Add custom title text.
         legend : Bool, default: True
             Show the legend based on the unique y-labels.
         figsize : (int, int), optional, default: (15, 10)
@@ -549,6 +562,8 @@ class pca():
         """
         if self.results['PC'].shape[1]>=3:
             fig, ax = self.scatter(y=y,
+                                   c=c,
+                                   s=s,
                                    d3=True,
                                    label=label,
                                    PC=PC, SPE=SPE,
@@ -561,6 +576,7 @@ class pca():
                                    legend=legend,
                                    figsize=figsize,
                                    visible=visible,
+                                   fig=fig,
                                    verbose=verbose)
         else:
             print('[pca] >Error: There are not enough PCs to make a 3d-plot.')
@@ -570,6 +586,7 @@ class pca():
     # Scatter plot
     def scatter(self,
                 y=None,
+                c=None,
                 s=50,
                 d3=False,
                 label=True,
@@ -578,12 +595,13 @@ class pca():
                 hotellingt2=False,
                 alpha_transparency=None,
                 gradient=None,
-                fontdict={'weight': 'normal', 'size': 12, 'ha': 'center', 'va': 'center'},
+                fontdict={'weight': 'normal', 'size': 12, 'ha': 'center', 'va': 'center', 'c': 'black'},
                 cmap='Set1',
                 title=None,
                 legend=True,
                 figsize=(20, 15),
                 visible=True,
+                fig=None,
                 verbose=3):
         """Scatter 2d plot.
 
@@ -591,6 +609,13 @@ class pca():
         ----------
         y : array-like, default: None
             Label for each sample. The labeling is used for coloring the samples.
+        c: list/array of RGB colors for each sample.
+            Color of samples in RGB colors.
+            [0,0,0]: If a single color is given, all samples get that color.
+        s: Int or list/array (default: 50)
+            Size(s) of the scatter-points.
+            [20, 10, 50, ...]: In case of list: should be same size as the number of PCs -> .results['PC']
+            50: all points get this size.
         d3 : Bool, default: False
             3d plot is created when True.
         label : Bool, default: True
@@ -610,11 +635,14 @@ class pca():
             '#FFFFFF'
         fontdict : dict.
             dictionary containing properties for the arrow font-text
-            {'weight': 'normal', 'size': 10, 'ha': 'center', 'va': 'center'}
+            {'weight': 'normal', 'size': 10, 'ha': 'center', 'va': 'center', 'c': 'black'}
         cmap : String, optional, default: 'Set1'
             Colormap. If set to None, no points are shown.
-        title : str, (default: None)
+        title : str, default: None
             Title of the figure.
+            None: Automatically create title text based on results.
+            '' : Remove all title text.
+            'title text' : Add custom title text.
         legend : Bool, default: True
             Show the legend based on the unique y-labels.
         figsize : (int, int), optional, default: (15, 10)
@@ -629,14 +657,20 @@ class pca():
         tuple containing (fig, ax)
 
         """
+        if c is None: c=[[0, 0, 0]]
         if (gradient is not None) and ((not isinstance(gradient, str)) or (len(gradient)!=7)): raise Exception('[pca]> Error: gradient must be of type string with Hex color or None.')
-        fontdict = {**{'weight': 'normal', 'size': 10, 'ha': 'center', 'va': 'center'}, **fontdict}
+        fontdict = _set_fontdict(fontdict)
+
         # Setup figure
-        fig = plt.figure(figsize=figsize)
-        if d3:
-            ax = fig.add_subplot(projection='3d')
+        if fig is None:
+            fig = plt.figure(figsize=figsize)
+            if d3:
+                ax = fig.add_subplot(projection='3d')
+            else:
+                ax = fig.add_subplot()
         else:
-            ax = fig.add_subplot()
+            ax = fig.axes[0]
+
         # fig, ax = plt.subplots(figsize=figsize, edgecolor='k')
         fig.set_visible(visible)
 
@@ -652,11 +686,12 @@ class pca():
 
         # Get the colors
         if cmap is None:
+            # Hide the scatterpoints by making them all white.
             getcolors = np.repeat([1., 1., 1.], len(y), axis=0).reshape(-1, 3)
         else:
             # Figure properties
             xyz, _ = scatterd._preprocessing(xs, ys, zs, y)
-            getcolors, fontcolor = scatterd.set_colors(xyz, y, None, [[0, 0, 0]], cmap, gradient=gradient)
+            getcolors, fontcolor = scatterd.set_colors(xyz, y, None, c, cmap, gradient=gradient)
 
         if hotellingt2 and ('y_bool' in self.results['outliers'].columns):
             Ioutlier1 = self.results['outliers']['y_bool'].values
@@ -668,7 +703,6 @@ class pca():
                 if g_ellipse is not None: ax.add_artist(g_ellipse)
 
         # Make scatter plot of all not-outliers
-        # Inormal = ~np.logical_or(Ioutlier1, Ioutlier2)
         uiy = np.unique(y)
 
         if (len(uiy)==len(y)) and (len(uiy)>=1000) and (label is not None):
@@ -682,9 +716,6 @@ class pca():
         else:
             for yk in uiy:
                 Iloc_sampl = (yk==y)
-                # Iloc_sampl = np.logical_and(Iloc_sampl, Inormal)
-                # Set color back to the mapped samples
-                # if yk=='mapped': Iloc_sampl[y=='mapped']=True
 
                 if d3:
                     ax.scatter(xs[Iloc_sampl], ys[Iloc_sampl], zs[Iloc_sampl], s=s + 10, label=yk, alpha=alpha_transparency, color=getcolors[Iloc_sampl, :])
@@ -732,6 +763,8 @@ class pca():
 
     def biplot(self,
                y=None,
+               c=None,
+               s=50,
                n_feat=None,
                d3=False,
                label=True,
@@ -741,12 +774,13 @@ class pca():
                alpha_transparency=None,
                gradient=None,
                color_arrow='r',
-               fontdict={'weight': 'normal', 'size': 12, 'ha': 'center', 'va': 'center'},
+               fontdict={'weight': 'normal', 'size': 12, 'ha': 'center', 'va': 'center', 'c': 'color_arrow'},
                cmap='Set1',
                title=None,
                legend=True,
                figsize=(15, 10),
                visible=True,
+               fig=None,
                verbose=3):
         """Create the Biplot.
 
@@ -761,6 +795,13 @@ class pca():
         ----------
         y : array-like, default: None
             Label for each sample. The labeling is used for coloring the samples.
+        c: list/array of RGB colors for each sample.
+            Color of samples in RGB colors.
+            [0,0,0]: If a single color is given, all samples get that color.
+        s: Int or list/array (default: 50)
+            Size(s) of the scatter-points.
+            [20, 10, 50, ...]: In case of list: should be same size as the number of PCs -> .results['PC']
+            50: all points get this size.
         n_feat : int, default: 10
             Number of features that explain the space the most, dervied from the loadings. This parameter is used for vizualization purposes only.
         d3 : Bool, default: False
@@ -784,18 +825,24 @@ class pca():
             color for the arrow.
             'r' (default)
         fontdict : dict.
-            dictionary containing properties for the arrow font-text
-            {'weight': 'normal', 'size': 10, 'ha': 'center', 'va': 'center'}
+            dictionary containing properties for the arrow font-text.
+            Note that the [c]olor: 'color_arrow' inherits the color used in color_arrow.
+            {'weight': 'normal', 'size': 10, 'ha': 'center', 'va': 'center', 'c': 'color_arrow'}
         cmap : String, optional, default: 'Set1'
             Colormap. If set to None, no points are shown.
-        title : str, (default: None)
+        title : str, default: None
             Title of the figure.
+            None: Automatically create title text based on results.
+            '' : Remove all title text.
+            'title text' : Add custom title text.
         legend : Bool, default: True
             Show the legend based on the unique y-labels.
         figsize : (int, int), optional, default: (15, 10)
             (width, height) in inches.
         visible : Bool, default: True
             Visible status of the Figure. When False, figure is created on the background.
+        fig : Object, Default: None
+            Figure Object.
         Verbose : int (default : 3)
             The higher the number, the more information is printed.
             Print to screen. 0: None, 1: Error, 2: Warning, 3: Info, 4: Debug, 5: Trace
@@ -810,15 +857,8 @@ class pca():
             * https://stackoverflow.com/questions/50796024/feature-variable-importance-after-a-pca-analysis/50845697#50845697
 
         """
-        if self.results['PC'].shape[1]<2: raise ValueError('[pca] >[Error] Requires 2 PCs to make 2d plot.')
-        if d3 and len(PC)<3: raise ValueError('[pca] >[Error] in case of biplot3d or d3=True, at least 3 PCs are required.')
-        if np.max(PC)>=self.results['PC'].shape[1]: raise ValueError('[pca] >[Error] PC%.0d does not exist!' %(np.max(PC) + 1))
-        if verbose>=3 and d3:
-            print('[pca] >Plot PC%.0d vs PC%.0d vs PC%.0d with loadings.' %(PC[0] + 1, PC[1] + 1, PC[2] + 1))
-        elif verbose>=3:
-            print('[pca] >Plot PC%.0d vs PC%.0d with loadings.' %(PC[0] + 1, PC[1] + 1))
-        if cmap is False: cmap=None
-        fontdict = {**{'weight': 'normal', 'size': 10, 'ha': 'center', 'va': 'center'}, **fontdict}
+        # Input checks
+        fontdict, cmap = _biplot_input_checks(self.results, PC, cmap, fontdict, d3, color_arrow, verbose)
 
         # Pre-processing
         y, topfeat, n_feat = self._fig_preprocessing(y, n_feat, d3)
@@ -845,9 +885,9 @@ class pca():
                 return None, None
             mean_z = np.mean(self.results['PC'].iloc[:, PC[2]].values)
             # zs = self.results['PC'].iloc[:,2].values
-            fig, ax = self.scatter3d(y=y, label=label, legend=legend, PC=PC, SPE=SPE, hotellingt2=hotellingt2, cmap=cmap, visible=visible, figsize=figsize, alpha_transparency=alpha_transparency, title=title, gradient=gradient)
+            fig, ax = self.scatter3d(y=y, label=label, legend=legend, PC=PC, SPE=SPE, hotellingt2=hotellingt2, cmap=cmap, visible=visible, figsize=figsize, alpha_transparency=alpha_transparency, title=title, gradient=gradient, fig=fig, c=c, s=s)
         else:
-            fig, ax = self.scatter(y=y, label=label, legend=legend, PC=PC, SPE=SPE, hotellingt2=hotellingt2, cmap=cmap, visible=visible, figsize=figsize, alpha_transparency=alpha_transparency, title=title, gradient=gradient)
+            fig, ax = self.scatter(y=y, label=label, legend=legend, PC=PC, SPE=SPE, hotellingt2=hotellingt2, cmap=cmap, visible=visible, figsize=figsize, alpha_transparency=alpha_transparency, title=title, gradient=gradient, fig=fig, c=c, s=s)
 
         # For vizualization purposes we will keep only the unique feature-names
         topfeat = topfeat.drop_duplicates(subset=['feature'])
@@ -876,11 +916,13 @@ class pca():
 
         # Plot the adjusted text labels to prevent overlap
         if len(texts)>0: adjust_text(texts)
-        if visible: plt.show()
-        return(fig, ax)
+        # if visible: plt.show()
+        return (fig, ax)
 
     def biplot3d(self,
                  y=None,
+                 c=None,
+                 s=50,
                  n_feat=None,
                  label=True,
                  PC=[0, 1, 2],
@@ -889,12 +931,13 @@ class pca():
                  alpha_transparency=1,
                  gradient=None,
                  color_arrow='r',
-                 fontdict={'weight': 'normal', 'size': 10, 'ha': 'center', 'va': 'center'},
+                 fontdict={'weight': 'normal', 'size': 10, 'ha': 'center', 'va': 'center', 'c': 'color_arrow'},
                  cmap='Set1',
                  title=None,
                  legend=True,
                  figsize=(15, 10),
                  visible=True,
+                 fig=None,
                  verbose=3):
         """Make biplot in 3d.
 
@@ -902,6 +945,13 @@ class pca():
         ----------
         y : array-like, default: None
             Label for each sample. The labeling is used for coloring the samples.
+        c: list/array of RGB colors for each sample.
+            Color of samples in RGB colors.
+            [0,0,0]: If a single color is given, all samples get that color.
+        s: Int or list/array (default: 50)
+            Size(s) of the scatter-points.
+            [20, 10, 50, ...]: In case of list: should be same size as the number of PCs -> .results['PC']
+            50: all points get this size.
         n_feat : int, default: 10
             Number of features that explain the space the most, dervied from the loadings. This parameter is used for vizualization purposes only.
         label : Bool, default: True
@@ -924,17 +974,23 @@ class pca():
             'r' (default)
         fontdict : dict.
             dictionary containing properties for the arrow font-text
-            {'weight': 'normal', 'size': 10, 'ha': 'center', 'va': 'center'}
+            Note that the [c]olor: 'color_arrow' inherits the color used in color_arrow.
+            {'weight': 'normal', 'size': 10, 'ha': 'center', 'va': 'center', 'c': 'color_arrow'}
         cmap : String, optional, default: 'Set1'
             Colormap. If set to None, no points are shown.
-        title : str, (default: None)
+        title : str, default: None
             Title of the figure.
+            None: Automatically create title text based on results.
+            '' : Remove all title text.
+            'title text' : Add custom title text.
         legend : Bool, default: True
             Show the legend based on the unique y-labels.
         figsize : (int, int), optional, default: (15, 10)
             (width, height) in inches.
         visible : Bool, default: True
             Visible status of the Figure. When False, figure is created on the background.
+        fig : Object, Default: None
+            Figure Object.
         Verbose : int (default : 3)
             The higher the number, the more information is printed.
             Print to screen. 0: None, 1: Error, 2: Warning, 3: Info, 4: Debug, 5: Trace
@@ -950,6 +1006,8 @@ class pca():
 
         fig, ax = self.biplot(y=y,
                               n_feat=n_feat,
+                              c=c,
+                              s=s,
                               d3=True,
                               label=label,
                               PC=PC,
@@ -964,12 +1022,13 @@ class pca():
                               legend=legend,
                               figsize=figsize,
                               visible=visible,
+                              fig=fig,
                               verbose=verbose)
 
-        return(fig, ax)
+        return (fig, ax)
 
     # Show explained variance plot
-    def plot(self, n_components=None, xsteps=None, title=None, visible=True, figsize=(15, 10), verbose=3):
+    def plot(self, n_components=None, xsteps=None, title=None, visible=True, figsize=(15, 10), fig=None, verbose=3):
         """Scree-plot together with explained variance.
 
         Parameters
@@ -979,14 +1038,19 @@ class pca():
             None: All PCs.
         xsteps : int, optional
             Set the number of xticklabels.
-        title : str, (default: None)
+        title : str, default: None
             Title of the figure.
+            None: Automatically create title text based on results.
+            '' : Remove all title text.
+            'title text' : Add custom title text.
         visible : Bool, default: True
             Visible status of the Figure
             True : Figure is shown.
             False: Figure is created on the background.
         figsize : (int, int)
             (width, height) in inches.
+        fig : Object, Default: None
+            Figure Object.
         Verbose : int (default : 3)
             The higher the number, the more information is printed.
             Print to screen. 0: None, 1: Error, 2: Warning, 3: Info, 4: Debug, 5: Trace
@@ -1011,7 +1075,11 @@ class pca():
         xtick_idx = np.arange(1, len(explvar) + 1)
 
         # Make figure
-        fig, ax = plt.subplots(figsize=figsize, edgecolor='k')
+        if fig is None:
+            fig, ax = plt.subplots(figsize=figsize, edgecolor='k')
+        else:
+            ax = fig.axes[0]
+        # Set visibility and plot
         fig.set_visible(visible)
         plt.plot(xtick_idx, explvarCum, 'o-', color='k', linewidth=1, label='Cumulative explained variance')
 
@@ -1042,7 +1110,7 @@ class pca():
             plt.show()
             plt.draw()
         # Return
-        return(fig, ax)
+        return (fig, ax)
 
     # Top scoring components
     def norm(self, X, n_components=None, pcexclude=[1]):
@@ -1093,7 +1161,7 @@ class pca():
         # Transform data
         out = np.repeat(np.mean(X, axis=1).reshape(-1, 1), X.shape[1], axis=1) + np.dot(score.values[:, ndims], coeff[:, ndims].T)
         # Return
-        return(out)
+        return out
 
     # Import example
     def import_example(self, data='titanic', verbose=3):
@@ -1289,7 +1357,7 @@ def hotellingsT2(X, alpha=0.05, df=1, n_components=5, param=None, verbose=3):
         Pcomb.append(stats.combine_pvalues(y_proba[i, :], method='fisher'))
 
     Pcomb = np.array(Pcomb)
-    outliers = pd.DataFrame(data={'y_proba':Pcomb[:, 1], 'y_score': Pcomb[:, 0], 'y_bool': Pcomb[:, 1] <= alpha})
+    outliers = pd.DataFrame(data={'y_proba': Pcomb[:, 1], 'y_score': Pcomb[:, 0], 'y_bool': Pcomb[:, 1] <= alpha})
     # Return
     return outliers, y_bools, param
 
@@ -1328,7 +1396,7 @@ def _explainedvar(X, method='pca', n_components=None, onehot=False, random_state
     #     percentExplVar = model.explained_variance_ratio_.cumsum()
 
     # Return
-    return(model, PC, loadings, percentExplVar)
+    return (model, PC, loadings, percentExplVar)
 
 
 # %% Store results
@@ -1396,6 +1464,9 @@ def import_example(data='titanic', verbose=3):
 # %%
 def _get_explained_variance(X, components):
     """Get the explained variance.
+
+    Description
+    -----------
     Get the explained variance from the principal components of the
     data. This follows the method outlined in [1] section 3.4 (Adjusted Total
     Variance). For an alternate approach (not implemented here), see [2].
@@ -1444,6 +1515,31 @@ def _get_explained_variance(X, components):
     # and the adjusted components
     Y = np.tensordot(X, proj_corrected_vecs.T, axes=(1, 0))
     YYT = np.tensordot(Y.T, Y, axes=(1, 0))
-    explained_variance = np.diag(YYT)/(n_samples-1)
+    explained_variance = np.diag(YYT) / (n_samples - 1)
 
-    return explained_variance
\ No newline at end of file
+    return explained_variance
+
+
+def _biplot_input_checks(results, PC, cmap, fontdict, d3, color_arrow, verbose):
+    # Check PCs
+    if results['PC'].shape[1]<2: raise ValueError('[pca] >[Error] Requires 2 PCs to make 2d plot.')
+    if d3 and len(PC)<3: raise ValueError('[pca] >[Error] in case of biplot3d or d3=True, at least 3 PCs are required.')
+    if np.max(PC)>=results['PC'].shape[1]: raise ValueError('[pca] >[Error] PC%.0d does not exist!' %(np.max(PC) + 1))
+    if verbose>=3 and d3:
+        print('[pca] >Plot PC%.0d vs PC%.0d vs PC%.0d with loadings.' %(PC[0] + 1, PC[1] + 1, PC[2] + 1))
+    elif verbose>=3:
+        print('[pca] >Plot PC%.0d vs PC%.0d with loadings.' %(PC[0] + 1, PC[1] + 1))
+    if cmap is False: cmap=None
+    # Set defaults in fontdict
+    fontdict =_set_fontdict(fontdict, color_arrow)
+    # Set font dictionary
+    # Return
+    return fontdict, cmap
+
+
+def _set_fontdict(fontdict, color_arrow=None):
+    color_arrow = 'black' if (color_arrow is None) else color_arrow
+    fontdict = {**{'weight': 'normal', 'size': 10, 'ha': 'center', 'va': 'center', 'c': color_arrow}, **fontdict}
+    if fontdict.get('c')=='color_arrow' and (color_arrow is not None):
+        fontdict['c'] = color_arrow
+    return fontdict
